#include <linux/compat.h>

#include "drmP.h"
#include "drm.h"
#include "mach64_drm.h"

/* DMA INIT */
typedef struct drm_mach64_init32 {
	int func;

	u32 sarea_priv_offset;
	int is_pci;

	int dma_mode;
	unsigned int fb_bpp;
	unsigned int front_offset, front_pitch;
	unsigned int back_offset, back_pitch;

	unsigned int depth_bpp;
	unsigned int depth_offset, depth_pitch;

	u32 fb_offset;
	u32 mmio_offset;
	u32 ring_offset;
	u32 buffers_offset;
	u32 agp_textures_offset;
} drm_mach64_init32_t;

static int compat_mach64_dma_init(struct file *file, unsigned int cmd,
				unsigned long arg)
{
	drm_mach64_init32_t init32;
	drm_mach64_init_t __user *init;
	int error;

	if (copy_from_user(&init32, (void __user *)arg, sizeof(init32)))
		return -EFAULT;

	init = compat_alloc_user_space(sizeof(*init));

	if (!access_ok(VERIFY_WRITE, init, sizeof(*init))
		|| __put_user(init32.func, &init->func)
		|| __put_user(init32.sarea_priv_offset, &init->sarea_priv_offset)
		|| __put_user(init32.is_pci, &init->is_pci)
		|| __put_user(init32.dma_mode, &init->dma_mode)
		|| __put_user(init32.fb_bpp, &init->fb_bpp)
		|| __put_user(init32.front_offset, &init->front_offset)
		|| __put_user(init32.front_pitch, &init->front_pitch)
		|| __put_user(init32.back_offset, &init->back_offset)
		|| __put_user(init32.back_pitch, &init->back_pitch)
		|| __put_user(init32.depth_bpp, &init->depth_bpp)
		|| __put_user(init32.depth_offset, &init->depth_offset)
		|| __put_user(init32.depth_pitch, &init->depth_pitch)
		|| __put_user(init32.fb_offset, &init->fb_offset)
		|| __put_user(init32.mmio_offset, &init->mmio_offset)
		|| __put_user(init32.ring_offset, &init->ring_offset)
		|| __put_user(init32.buffers_offset, &init->buffers_offset)
		|| __put_user(init32.agp_textures_offset, &init->agp_textures_offset)
		)
		return -EFAULT;

	error =  drm_ioctl(file->f_path.dentry->d_inode, file,
			 DRM_IOCTL_MACH64_INIT, (unsigned long)init);
	return error;

}

/* DMA CLEAR */

typedef struct drm_mach64_clear32 {
	unsigned int flags;
	int x, y, w, h;
	unsigned int clear_color;
	unsigned int clear_depth;
} drm_mach64_clear32_t;

static int compat_mach64_dma_clear(struct file *file, unsigned int cmd,
				unsigned long arg)
{
	drm_mach64_clear32_t clr32;
	drm_mach64_clear_t __user *clr;

	if (copy_from_user(&clr32, (void __user *)arg, sizeof(clr32)))
		return -EFAULT;

	clr = compat_alloc_user_space(sizeof(*clr));

	if (!access_ok(VERIFY_WRITE, clr, sizeof(*clr))
		|| __put_user(clr32.flags, &clr->flags)
		|| __put_user(clr32.x, &clr->x)
		|| __put_user(clr32.y, &clr->y)
		|| __put_user(clr32.w, &clr->w)
		|| __put_user(clr32.h, &clr->h)
		|| __put_user(clr32.clear_color, &clr->clear_color)
		|| __put_user(clr32.clear_depth, &clr->clear_depth))
		return -EFAULT;

	return drm_ioctl(file->f_path.dentry->d_inode, file,
			 DRM_IOCTL_MACH64_CLEAR, (unsigned long)clr);
}

/* DMA VERTEX */
typedef struct drm_mach64_vertex32 {
	int prim;
	u32 buf;		/* Address of vertex buffer */
	u32 used;	/* Number of bytes in buffer */
	int discard;		/* Client finished with buffer? */
} drm_mach64_vertex32_t;

static int compat_mach64_dma_vertex(struct file *file, unsigned int cmd,
				unsigned long arg)
{
	drm_mach64_vertex32_t vertex32;
	drm_mach64_vertex_t __user *vertex;

	if (copy_from_user(&vertex32, (void __user *)arg, sizeof(vertex32)))
		return -EFAULT;

	vertex = compat_alloc_user_space(sizeof(*vertex));

	if (!access_ok(VERIFY_WRITE, vertex, sizeof(*vertex))
		|| __put_user(vertex32.prim, &vertex->prim)
		|| __put_user((unsigned int __user *)(unsigned long)vertex32.buf, &vertex->buf)
		|| __put_user(vertex32.used, &vertex->used)
		|| __put_user(vertex32.discard, &vertex->discard))
		return -EFAULT;

	return drm_ioctl(file->f_path.dentry->d_inode, file,
			 DRM_IOCTL_MACH64_VERTEX, (unsigned long)vertex);
}

/* DMA BLIT */
typedef struct drm_mach64_blit32 {
	u32 buf;
	int pitch;
	int offset;
	int format;
	unsigned short x, y;
	unsigned short width, height;
} drm_mach64_blit32_t;

static int compat_mach64_dma_blit(struct file *file, unsigned int cmd,
				unsigned long arg)
{
	drm_mach64_blit32_t blit32;
	drm_mach64_blit_t __user *blit;

	if (copy_from_user(&blit32, (void __user *)arg, sizeof(blit32)))
		return -EFAULT;

	blit = compat_alloc_user_space(sizeof(*blit));

	if (!access_ok(VERIFY_WRITE, blit, sizeof(*blit))
		|| __put_user((unsigned int __user *)(unsigned long)blit32.buf, &blit->buf)
		|| __put_user(blit32.pitch, &blit->pitch)
		|| __put_user(blit32.offset, &blit->offset)
		|| __put_user(blit32.format, &blit->format)
		|| __put_user(blit32.x, &blit->x)
		|| __put_user(blit32.y, &blit->y)
		|| __put_user(blit32.width, &blit->width)
		|| __put_user(blit32.height, &blit->height))
		return -EFAULT;

	return drm_ioctl(file->f_path.dentry->d_inode, file,
			 DRM_IOCTL_MACH64_BLIT, (unsigned long)blit);
}



/* GETPARAM */
typedef struct drm_mach64_getparam32 {
	int param;
	u32 value;
} drm_mach64_getparam32_t;

static int compat_mach64_get_param(struct file *file, unsigned int cmd,
			       unsigned long arg)
{
	drm_mach64_getparam32_t getparam32;
	drm_mach64_getparam_t __user *getparam;

	if (copy_from_user(&getparam32, (void __user *)arg, sizeof(getparam32)))
		return -EFAULT;

	getparam = compat_alloc_user_space(sizeof(*getparam));
	if (!access_ok(VERIFY_WRITE, getparam, sizeof(*getparam))
	    || __put_user(getparam32.param, &getparam->param)
	    || __put_user((void __user *)(unsigned long)getparam32.value,
			  &getparam->value))
		return -EFAULT;

	return drm_ioctl(file->f_path.dentry->d_inode, file,
			 DRM_IOCTL_MACH64_GETPARAM, (unsigned long)getparam);
}


drm_ioctl_compat_t *mach64_compat_ioctls[] = {
	[DRM_MACH64_INIT] = compat_mach64_dma_init,
	[DRM_MACH64_CLEAR] = compat_mach64_dma_clear,
#if 0
	/* ? not needed */
	[DRM_MACH64_SWAP] = compat_mach64_dma_swap,
	[DRM_MACH64_IDLE] = compat_mach64_dma_idle,
	[DRM_MACH64_RESET] = compat_mach64_engine_reset,
#endif
	[DRM_MACH64_VERTEX] = compat_mach64_dma_vertex,
	[DRM_MACH64_BLIT] = compat_mach64_dma_blit,
#if 0
	/* ? not needed */
	[DRM_MACH64_FLUSH] = compat_mach64_dma_flush,
#endif
	[DRM_MACH64_GETPARAM] = compat_mach64_get_param,
};

/**
 * Called whenever a 32-bit process running under a 64-bit kernel
 * performs an ioctl on /dev/dri/card<n>.
 *
 * \param filp file pointer.
 * \param cmd command.
 * \param arg user argument.
 * \return zero on success or negative number on failure.
 */
long mach64_compat_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	unsigned int nr = DRM_IOCTL_NR(cmd);
	drm_ioctl_compat_t *fn = NULL;
	int ret;

	printk(KERN_ERR "ioctl %d\n", nr);
	if (nr < DRM_COMMAND_BASE)
		return drm_compat_ioctl(filp, cmd, arg);

	if (nr < DRM_COMMAND_BASE + DRM_ARRAY_SIZE(mach64_compat_ioctls))
		fn = mach64_compat_ioctls[nr - DRM_COMMAND_BASE];

	lock_kernel();		/* XXX for now */
	if (fn != NULL) {
		ret = (*fn) (filp, cmd, arg);
		printk(KERN_ERR "Handled by %s returned %d\n", __FILE__, ret);
	}
	else {
		ret = drm_ioctl(filp->f_path.dentry->d_inode, filp, cmd, arg);
		printk(KERN_ERR "Not handled by %s return %d\n", __FILE__, ret);
	}
	unlock_kernel();

	return ret;
}
